@@ -1,11 +1,11 @@
 gsm <- 
-  function(formula, family = gaussian, data, weights, 
-           types = NULL, tprk = TRUE, knots = NULL, update = TRUE, 
-           spar = NULL, lambda = NULL, control = list(),
-           method = c("GCV", "OCV", "GACV", "ACV", "PQL", "AIC", "BIC")){
-    # generalized semiparametric model
+  function(formula, family = gaussian, data, weights, types = NULL, tprk = TRUE, 
+           knots = NULL, skip.iter = TRUE, spar = NULL, lambda = NULL, control = list(),
+           method = c("GCV", "OCV", "GACV", "ACV", "PQL", "AIC", "BIC"),
+           xrange = NULL, thetas = NULL, mf = NULL){
+    # generalized smooth model
     # Nathaniel E. Helwig (helwig@umn.edu)
-    # Updated: 2020-07-24
+    # Updated: 2022-03-30
     
     
     #########***#########   CHECKS   #########***#########
@@ -39,11 +39,13 @@ gsm <-
     #########***#########   FORMULA   #########***#########
     
     # create model frame
-    mf <- match.call()
-    m <- match(c("formula","data","weights"),names(mf),0L)
-    mf <- mf[c(1L, m)]
-    mf[[1L]] <- as.name("model.frame")
-    mf <- eval(mf, parent.frame())
+    if(is.null(mf)){
+      mf <- match.call()
+      m <- match(c("formula","data","weights"),names(mf),0L)
+      mf <- mf[c(1L, m)]
+      mf[[1L]] <- as.name("model.frame")
+      mf <- eval(mf, parent.frame())
+    }
     mt <- attr(mf, "terms")                 # mt contains model info and terms 
     et <- attr(mt,"factors")                # et is effects table
     mfdim <- dim(et)                        # dim of effects table
@@ -74,8 +76,22 @@ gsm <-
     if(ncol(yvar) > 1){ stop("Response must be unidimensional (vector).") }
     
     
+    #########***#########   WEIGHTS   #########***#########
+    xlist <- as.list(mf[,-1,drop=FALSE])
+    if(any(names(xlist) == "(weights)")) {
+      no.weights <- FALSE
+      xlist <- xlist[-which(names(xlist) == "(weights)")]
+      weights <- as.numeric(mf[,"(weights)"])
+      if(any(weights < 0)) stop("Input 'weights' must be non-negative.")
+      if(family$family != "binomial") weights <- weights * sum(weights > 0) / sum(weights)
+    } else {
+      no.weights <- TRUE
+      weights <- rep(1, nobs)
+    }
+    
+    
     #########***#########   GET TYPES   #########***#########
-    chty <- check_type(mf, types)
+    chty <- check_type(mf, types, xrange)
     
     
     #########***#########   GET KNOTS   #########***#########
@@ -84,24 +100,13 @@ gsm <-
     
     
     #########***#########   GET RKHS   #########***#########
-    xlist <- as.list(mf[,-1,drop=FALSE])
-    if(any(names(xlist) == "(weights)")) {
-      xlist <- xlist[-which(names(xlist) == "(weights)")]
-    }
-    rkhs <- build_rkhs(x = xlist, type = chty$type, knots = knot, xrng = chty$xrng)
+    rkhs <- build_rkhs(x = xlist, type = chty$type, knots = knot, 
+                       xrng = chty$xrng, xlev = chty$xlev)
     
     
     #########***#########   DESIGN & PENALTY   #########***#########
-    depe <- build_depe(Etab = et, rkhs = rkhs, tprk = tprk)
-    if(missing(weights)){
-      no.weights <- TRUE
-      depe$weights <- rep(1, nobs)
-    } else {
-      no.weights <- FALSE
-      depe$weights <- c(mf[,"(weights)"])
-      if(any(depe$weights < 0)) stop("Input 'weights' must be non-negative.")
-      if(family$family != "binomial") depe$weights <- depe$weights * sum(depe$weights > 0) / sum(depe$weights)
-    }
+    depe <- build_depe(Etab = et, rkhs = rkhs, tprk = tprk, thetas = thetas)
+    depe$weights <- weights
     
     
     #########***#########   FIT MODEL   #########***#########
@@ -114,7 +119,7 @@ gsm <-
     # use Algorithm 3.2 from Gu and Wahba (1991)
     nullindx <- 1:fit$nsdf
     allzero <- (sum(abs(fit$coefficients[-nullindx])) == 0)
-    if(update && (length(depe$thetas) > 1L) && !allzero){
+    if((length(depe$thetas) > 1L) && is.null(thetas) && !allzero){
       
       # rebuild design and penalty
       if(tprk){
@@ -126,18 +131,61 @@ gsm <-
                             Jcoef = fit$coef[-nullindx],
                             depe = depe, tprk = FALSE)
       }
-      if(missing(weights)){
-        depe$weights <- rep(1, nobs)
-      } else {
-        depe$weights <- as.numeric(mf[,"(weights)"])
-      }
+      depe$weights <- weights
       
       # refit model
       fit <- fit_gsm(y = yvar, depe = depe, lambda = lambda,
                      tprk = tprk, control = control, method = method,
                      family = family)
       
-    } # end if(update && (length(depe$thetas) > 1L) && !allzero)
+      # deep tuning?
+      if(!skip.iter){
+        opt <- nlm(f = tune.deep.gsm, p = log(depe$thetas),
+                   spar = fit$spar, y = yvar, Etab = et, rkhs = rkhs, 
+                   weights = depe$weights, tprk = tprk, method = method, 
+                   family = fit$family, control = control, gradtol = control$tol, 
+                   iterlim = control$iterlim, print.level = control$print.level)
+        thetas <- exp(opt$estimate)
+        names(thetas) <- names(depe$thetas)
+        depe <- build_depe(Etab = et, rkhs = rkhs, tprk = tprk, thetas = thetas)
+        depe$weights <- weights
+        fit <- fit_gsm(y = yvar, depe = depe, lambda = fit$lambda,
+                       tprk = tprk, control = control, method = method,
+                       family = family)
+        fit$iter <- c(irpls = fit$iter, nlm = opt$iter)
+        if(family$family == "NegBin" && !family$fixed.theta){
+          iter.out <- 0L
+          vtol.out <- control$epsilon.out + 1
+          obj.old <- as.numeric(opt$minimum)
+          while(iter.out < control$maxit.out & vtol.out > control$epsilon.out){
+            mu <- family$linkinv(cbind(depe$K, depe$J) %*% fit$coefficients)
+            theta.hat <- theta.mle(y = yvar, mu = mu, wt = depe$weights)
+            family <- NegBin(theta = theta.hat, link = family$link)
+            opt <- nlm(f = tune.deep.gsm, p = log(depe$thetas),
+                       spar = fit$spar, y = yvar, Etab = et, rkhs = rkhs,
+                       weights = depe$weights, tprk = tprk, method = method,
+                       family = family, control = control, 
+                       gradtol = control$tol, iterlim = control$iterlim, 
+                       print.level = control$print.level)
+            thetas <- exp(opt$estimate)
+            names(thetas) <- names(depe$thetas)
+            vtol.out <- abs( (opt$minimum - obj.old) / (obj.old + 1e-4) )
+            iter.out <- iter.out + 1L
+            obj.old <- opt$minimum
+            depe <- build_depe(Etab = et, rkhs = rkhs, tprk = tprk, thetas = thetas)
+            depe$weights <- weights
+            fit <- fit_gsm(y = yvar, depe = depe, lambda = fit$lambda,
+                           tprk = tprk, control = control, method = method,
+                           family = family)
+            fit$iter <- c(inner = fit$iter, outter = attr(theta.hat, "iter"), nlm = opt$iter)
+          } # end while(iter.out < control$maxit.out & vtol.out > control$epsilon.out)
+          family$fixed.theta <- FALSE
+          attr(fit$family$theta, "SE") <- attr(theta.hat, "SE")
+          attr(fit$family$theta, "iter") <- attr(theta.hat, "iter")
+        } # end if(family$family == "NegBin" && !family$fixed.theta)
+      } # end if(!skip.iter)
+      
+    } # end if((length(depe$thetas) > 1L) && !allzero)
     
     
     #########***#########   RETURN RESULTS   #########***#########
@@ -154,7 +202,8 @@ gsm <-
     # fit specs
     specs <- list(knots = knot, thetas = depe$thetas, 
                   xrng = chty$xrng, xlev = chty$xlev,
-                  tprk = tprk)
+                  tprk = tprk, skip.iter = skip.iter,
+                  control = control)
     
     # collect results
     fit$specs <- specs
@@ -163,6 +212,7 @@ gsm <-
     fit$terms <- colnames(et)
     fit$method <- method
     fit$formula <- formula
+    fit$weights <- if(!no.weights) weights
     fit$call <- match.call()
     fit$family <- family
     fit <- fit[c(4:length(fit), 1:3)]
